#!/usr/bin/env python
#
# PkgTrackInst.InstallSession - Tracks installation log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: InstallSession.py 478 2005-05-01 14:42:01Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx
from PkgTrackInst.Trackers import (
        InstFile, InstDirectory, InstSymbolicLink, TrackRoot)

class InstallSession(object):
    def __init__(self):
        self.fdtrack = {}

    def feed(self, line):
        rawargs, ret = line.rsplit('#', 1)
        args = rawargs.strip().split('\t')

        fname = args[1]
        del args[1]
        args[0] = eval(args[0]) # args[0] is integer return.
        handler_name = 'hdl_' + fname

        if not hasattr(self, handler_name):
            errx("function `%s' isn't supported in %s." % (fname, sys.argv[0]))

        getattr(self, handler_name)(*args)

    def new_file(self, path):
        TrackRoot[path] = InstFile(created=True)

    def new_directory(self, path):
        TrackRoot[path] = InstDirectory(created=True)

    def new_symlink(self, src, dst):
        TrackRoot[dst] = InstSymbolicLink(src, created=True)

    def move(self, src, dst):
        dirname, basename = os.path.split(src)
        TrackRoot[dirname].move(basename, dst)

    def delete(self, path):
        del TrackRoot[path]

    def set_mode(self, path, mode, symlink=False):
        TrackRoot[path].chmod(eval(mode) & 03777, symlink)

    def set_owner(self, path, uid, gid, symlink=False):
        TrackRoot[path].chown(eval(uid), eval(gid), symlink)

    # -*- libc log handlers -*-

    def hdl_chmod(self, ret, path, mode):
        if ret != 0:
            self.set_mode(path, mode)

    def hdl_chown(self, ret, path, uid, gid):
        if ret != 0:
            self.set_owner(path, uid, gid, symlink=False)

    def hdl_creat(self, ret, path):
        return self.hdl_open(ret, path)

    def hdl_fchmod(self, ret, fd, mode):
        if ret != 0:
            if fd in self.fdtrack:
                self.set_mode(self.fdtrack[fd], mode)
            else:
                errx("fchmod() call for untracked file descriptor.")

    def hdl_fchown(self, ret, fd, uid, gid):
        if ret != 0:
            if fd in self.fdtrack:
                self.set_owner(self.fdtrack[fd], uid, gid)
            else:
                errx("fchown() call for untracked file descriptor.")

    def hdl_fopen(self, ret, path):
        if ret != 0:
            self.new_file(path)

    def hdl_ftruncate(self, ret, fd, length):
        pass # no need to handle

    def hdl_lchown(self, ret, path, uid, gid):
        if ret != 0:
            self.set_owner(path, uid, gid, symlink=True)

    def hdl_link(self, ret, src, dst):
        if ret == 0:
            # copy filename part from src if dst is a directory
            if os.path.isdir(dst):
                namepart = os.path.split(src)[-1]
                dst = os.path.join(dst, namepart)
            self.new_file(dst)

    def hdl_mkdir(self, ret, path):
        if ret == 0:
            self.new_directory(path)

    def hdl_open(self, ret, path):
        if ret > 0:
            self.new_file(path)
            self.fdtrack[ret] = path

    def hdl_rename(self, ret, src, dst):
        if ret == 0:
            self.move(src, dst)

    def hdl_rmdir(self, ret, path):
        if ret == 0:
            self.delete(path)

    def hdl_symlink(self, ret, src, dst):
        if ret == 0:
            self.new_symlink(src, dst)

    def hdl_truncate(self, ret, path, length):
        if ret == 0:
            self.new_file(path)

    def hdl_unlink(self, ret, path):
        if ret == 0:
            self.delete(path)
