#!/usr/bin/env python
#
# PkgTrackInst.PlistWriter - Writes pkg-plist based on install log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: PlistWriter.py 478 2005-05-01 14:42:01Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
import time, re
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx, echo_msg
from PkgTrackInst.Trackers import InstFile, InstDirectory, TrackRoot

SUBST_IGNORE = ('PREFIX', 'PORTDOCS', 'LOCALBASE', 'X11BASE')

class PlistWriter(object):

    def __init__(self):
        self.plist_subst = []
        self.prefix = self.get_make_env('PREFIX')
        if options.substitute:
            self.build_sub_list()

    def get_output(self, cmd):
        f = os.popen(cmd, 'r')
        r = f.read()
        status = f.close()
        if status:
            print repr(status)
            errx("command `%s' exited with error code %d" % (cmd, status))
        return r

    def get_make_env(self, varname):
        return self.get_output('make -V ' + varname).strip()

    def build_sub_list(self):
        pat_subst = re.compile('([A-Za-z_0-9]+)=("([^"]*)"|[^"]\S*)')
        substlist = self.get_make_env('PLIST_SUB')

        for var, raw, stripped in pat_subst.findall(substlist):
            if var not in SUBST_IGNORE:
                self.plist_subst.append((stripped or raw, var))
        self.plist_subst.sort(reverse=True)

    def write(self):
        if os.path.exists('pkg-plist'):
            os.rename('pkg-plist', 'pkg-plist.bak')
        self.generate_plist(open('pkg-plist', 'w'))

    def relpath(self, fullpath):
        prefix = self.prefix.split(os.sep)
        full = fullpath.split(os.sep)
        if full[:len(prefix)] == prefix:
            repled = os.sep.join(full[len(prefix):])
            for orig, subst in self.plist_subst:
                if repled.startswith(orig+'/'):
                    return '%%' + subst + '%%' + repled[len(orig):]
            return repled
        else:
            return fullpath

    def generate_plist(self, out):
        echo_msg('Generating pkg-plist')

        fileents, dirents = [], []
        def gather_entries(ent):
            if ent.created:
                if isinstance(ent, InstDirectory):
                    dirents.append(ent)
                else:
                    fileents.append(ent)
        TrackRoot.walk(gather_entries)

        fileents.sort(key=lambda ent: ent.abspath)
        dirents.sort(key=lambda ent: ent.abspath)

        for ent in fileents:
            print >> out, self.relpath(ent.abspath)
        for ent in reversed(dirents):
            print >> out, '@dirrm ' + self.relpath(ent.abspath)
